/*-------------------- bisearch.cc (Binary Search) -------------------
  A binary search is more efficient than a linear search.
  It achieves this efficiency by assuming that the array has already
  	been sorted, then repeating dividing the search space in two
        (hence "binary") until either finding the desired element
        or giving up.
  Programmer: Glenn D. Blank  
  For chapter 11 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
--------------------------------------------------------------------*/
#include "\umach\cpp\libfiles\array.h"
const int NotFound = -1; 	//returned if search doesn't find key
int steps; //count number of iterations in BinarySearch()

int BinarySearch(int key,const IntArray& sorted,int last)
//find key item in sorted array between subscripts 0 and last
//returns either index of found item OR -1 indicating not found
{ int first=0;	//first index starts at 0 but may move with search
  while (first <= last)
  { int middle = (first + last) / 2; //find middle
    steps++;							    //tracking steps through loop
    if (sorted[middle] == key)       //match?
    return middle;              //return index #
    else if (sorted[middle] < key)   //key above middle?
           first = middle + 1;       //move first past middle
         else last = middle - 1;     //move last below middle
  }
  return NotFound;  //key is not in sorted array
}
#include <iostream.h>

void main()	//initialize an IntArray and test BinarySearch
{ IntArray ia(100); //user-supplied list of Lstrings
  const int fillThisMany=80; //insert this many values in ia

  //Create a partially filled array (ia.size() > fillThisMany)
  int i=0;
  while (i <= fillThisMany)
  //invariant: 0 < i <= fillThisMany
  { ia[i] = (i*2)+1;	//put an odd number in each element
    i++;
  }

  //Test BinarySearch
  cout << "Testing BinarySearch.  Press Ctrl-z then Enter to stop testing.\n";
  //Let the user supply an integer to test BinarySearch
  cout << "Type in an integer to test BinarySearch: ";
  int userInput; //user enters an integer
  cin >> userInput;
  while (!cin.eof())
  { steps=0;	//track # of iteration steps in BinarySearch()
    i = BinarySearch(userInput,ia,fillThisMany);
    if (i == NotFound)	cout << userInput << " not found ";
    else cout << "found " << userInput << " at index " << i;
    cout << " in " << steps << " iteration steps" << endl;
    cout << "Type in an integer to test BinarySearch: ";
    cin >> userInput;
  }
  cout << "Bye!" << endl;
}
