/*---------------------------- libtest.cpp -----------------------------
Demonstrates classes in Lookout class library introduced in chapter 5:
  Note (musical notes) and Lstring (strings of characters).
  A Borland IDE project file is available to create his program:
     in BCW, select Project, then Open Project, then libtest.ide.
  When compiling with Borland C++ 5.0 IDE:
  	Create a new project (File, New, Project) called libtest.
	Use Target Expert to set project with these settings:
	DOS STANDARD platform, LARGE target model, BGI on.
	Add LOOKOUT.LIB to your project (using the Insert key).
  The multimedia for chapter 5 explains this procedure in detail:
     in "UM", click on the Find button and look for "Lookout library".
  Programmer: Glenn D. Blank
  For chapter 5 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
---------------------------------------------------------------------*/
#include <iostream.h>
//Header files are in the umach\cpp\libfiles directory.
#include "\umach\cpp\libfiles\Lstring.h"
#include "\umach\cpp\libfiles\note.h"
#include "\umach\cpp\libfiles\shapes.h"
#include "\umach\cpp\libfiles\random.h"

void main()
{
  //class Note plays musical notes through a PC's speaker (not sound card)
  //Construct some notes
  cout << "Demonstrating musical Note class...." << endl;
  Note C("C");		//middle C is octave 4, the default octave
  Note E("E"); Note G("G");
  Note HiC("C", 5);	//C an octave above middle C

  //manipulate duration of Notes and play them
  C.eighth();
  C.triplet();
  C.play();	//play the sound of this Note for its duration
  C.show();	//show Lstring representation of Note

  //play rest of simple tune
  E.eighth();  E.triplet();   E.play();   E.show();
  G.eighth();  G.triplet();   G.play();   G.show();
  HiC.quarter();   HiC.triplet();  HiC.play();  HiC.show();
  G.play();  G.show();
  HiC.play(); HiC.show();

  //Construct a few Lstrings
  cout << "\n\nDemonstrating Lstring class...." << endl;
  Lstring s1;	   //default constructor is empty CSTRING, ""
  Lstring s2="xyz"; //providing initial value, "xyz"
  Lstring s3=s2;   //copy constructor copies value from another Lstring

  //display two Lstrings with initial values
  cout << "s1=" << s1 << endl;
  cout << "s2=" << s2 << endl;
  cout << "s3=" << s3 << endl;

  //read a value (up to a newline)
  cout << "Enter a string then press return:";
  cin >> s3;
  cout << "s3=" << s3 << endl;

  //length() and size() compute length of Lstring 
  cout << s1 << "'s length()=" << s1.length() << endl;
  cout << s2 << "'s length()=" << s2.length() << endl;
  cout << s3 << "'s size()=" << s3.size() << endl;

  //assignment copies a value into an Lstring
  s1 = s2;
  cout << "s1's value after s1=s2: " << s1 << endl;

  //concatenation assembles an Lstring out of two other Lstrings
  s1 = s2 + s3;
  cout << "After concatenation: " << s1 << endl;
  cout << s2 + "+" + s3 << endl;  //can concatenate CSTRINGS, too

  //find() searches for a start of an Lstring within an Lstring
  // find() returns a character index, where first character is 0
  s1 = "George Washington";
  int g = s1.find("G");	      //find 'G' in s1
  cout << "G is at character " << g << " in " << s1 << endl;
  int space = s1.find(" ");   //find space in s1
  cout << "Space is at character " << space << " in " << s1 << endl;
  //find() takes a second argument specifying where to start search
  int g2 = s1.find("g",space+1); //find 'g' past space, using 2nd arg
  cout << "'g' is at character " << g2 << " in Washington" << endl;
  //find() returns -1 to indicate failure
  int m = s1.find("Martha"); 	//attempt to find "Martha" in s1
  cout << m << " is result of s1.find(\"Martha\");" << endl;

  //substr() returns a subString of an Lstring
  s2 = s1.substr(0,space-1);  			//returns first name
  s3 = s1.substr(space+1,s1.length()); //returns last name
  cout << "First name=" << s2 << " Last name=" << s3 << endl;
  cout << "Take a look at the code to see what just happened!" << endl;
}
