/*------------------------ date.cc -------------------------------
Given a date in spelled-out format, e.g., June 22, 1990,
	produce the corresponding date in numeric format, e.g., 6/22/90
  Uses functions to decompose the problem.
  To create this program, you must use a multi-file project:
	see the comments in datetest.cc for details.
  Programmer: Glenn D. Blank  
  For chapter 9 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
------------------------------------------------------------------*/
#include <iostream.h>
#include <assert.h>
#include "..\lib\Lstring.h"

int getNumMonth(Lstring month)
{ //Given a month in Lstring form, return it in numeric form
  if (month=="January")        return 1;
  else if (month=="February")  return 2;
  else if (month=="March")     return 3;
  else if (month=="April")     return 4;
  else if (month=="May")       return 5;
  else if (month=="June")      return 6;
  else if (month=="July")      return 7;
  else if (month=="August")    return 8;
  else if (month=="September") return 9;
  else if (month=="October")   return 10;
  else if (month=="November")  return 11;
  else if (month=="December")  return 12;
  else //implicit postcondition
  { cout << "Misspelt month " << month << endl;
    assert(0);	//postcondition fails
    return 0;   //to satisfy function return type
  }
}

Lstring getDecadeYear(Lstring year)
{ //Given a century year (e.g., 1990) return a decade year (90)
  return year.substr(3,4);
}

void getParts(Lstring inDate,Lstring& month,Lstring& day,Lstring& year)
{ //Given a spelled-out date, break it up into three subLstrings
  // e.g., break "June 22, 1990" up into "June" "22" and "1990"
  //Use pass by reference to return the three subparts

  //preconditions: well-formed inDate
  assert(inDate.find(" ") != -1); //precondition: can find a month Lstring
  assert(inDate.find(",") != -1); //precondition: can find a day Lstring

  //find month subLstring, between start of inDate and blank space
  int endMonth = inDate.find(" "); 		//look for space after month
  month = inDate.substr(0,endMonth-1); //month between start of Date and ' '

  //find day subLstring, between space and comma
  int endDay=inDate.find(",");					//look for comma after day
  day = inDate.substr(endMonth+1,endDay-1);  //day between ' ' and ','

  //find year subLstring, between comma and end of inDate
  year=inDate.substr(endDay+1,inDate.length());  //year between ',' and end

  //postconditions: parts are in valid ranges
  assert(day >= "1" && day <= "31");	     //valid day
  assert(year >= " 1900" && year <= " 2099");  //this century or next
}

void getDate()
{
  Lstring userDate; 			//user supplied spelled out date

  //prompt user and read in a spelled out date
  cout << "Enter a spelled out date:";
  cin >> userDate;

  //Break userDate up into subLstrings
  Lstring month,day,year;	//subLstrings of userDate
  getParts(userDate,month,day,year);

  //convert month to numeric representation
  int numMonth = getNumMonth(month);

  //get decade part of year
  Lstring decadeYr = getDecadeYear(year);

  //display numeric representation of Date with slashes
  cout << numMonth << '/' << day << '/' << decadeYr << endl;
}


