#ifndef ARRAY_H
#define ARRAY_H
/* Arrays for C++--: IntArray, FloatArray, DoubleArray, StringArray
 * Authors: Jesse Thilo and Glenn Blank for The Universal Machine
 *
 * Constructors:
 *   FloatArray A(10);   Construct array of float A with bounds 0..9
 *                         and elements initialized to 0.0
 *   IntArray B(7,11);   Construct array of int B with bounds 0..6
			   and elements initialized to 7
 *   StringArray C(10);  Construct array of Lstring C with bounds 0..9
 *                         and elements initialized to ""
 *   IntArray D(B);      Copy size and contents of IntArray B into D
 * Members:
 *   A.size() or A.length()  Number of elements in array A
 *   A[i]		     Subscript operator accesses array element 
 *   A.sub(3)              Alternative syntax for A[i] which provides
 *	                   better error messages at run-time.
 *			   Lookout converts A[i] into A.sub(i).
*/

#include <stdlib.h>
#include <iostream.h>
#include "..\libfiles\Lstring.h"

class IntArray {  //an array of elements of type int
public:
  IntArray(int sz,int init=0); //sz elements with init value
  IntArray(IntArray &n);       //copy constructor 
  int size() const;	       //current size or capacity
  int length() const;	       //synonym of size()
  int& _sub(char*,int,int) const; //For subscripting bounds checking
  
#ifndef LOOKOUT 
  ~IntArray();  //destructor cleans up memory allocated by constructor
  int& operator[](int);	  //subscript operator returns int element
  const int& operator[](int) const; //const version of subscript
private:
  int *element;	 //pointer to a block of int
  int sz;	 //size or capacity 	
#endif
};

class FloatArray {  //an array of elements of type float
public:
  FloatArray(int sz,float init=0.0); //sz elements with init
  FloatArray(FloatArray &n);         //copy constructor
  int size() const;	             //current size or capacity
  int length() const;	             //synonym of size() 
  float& _sub(char*, int, int) const;  //for subscript bounds checking
#ifndef LOOKOUT
  ~FloatArray();           //destructor cleans up memory
  float& operator[](int);  //subscript operator returns float element
  const float& operator[](int) const; //const version of subscript

private:
  float *element; //pointer to a block of float
  int sz;	  //size or capacity 	
#endif
};

class DoubleArray { //an array of elements of type double
public:
  DoubleArray(int sz,double init=0.0); //sz elements with init value
  DoubleArray(DoubleArray &n);         //copy constructor
  int size() const;	               //current size or capacity
  int length() const;	               //synonym of size() 
#ifndef LOOKOUT
  ~DoubleArray();          //destructor cleans up memory
  double& operator[](int); //subscript operator returns double element
  const double& operator[](int) const; //const version of subscript
  double& _sub(char*, int, int) const;  //for subscript bounds checking
private:
  double *element; //pointer to a block of double
  int sz;	   //size or capacity 	
#endif
};

// #include "Lstring.h"

class StringArray { //an array of elements of type Lstring
public:
  StringArray(int sz,Lstring init=""); //sz elements with init
  StringArray(StringArray &);          //copy constructor
  int size() const;	               //current size or capacity
  int length() const;	               //synonym of size() 
  Lstring& _sub(char*,int,int) const;  //for subscript bounds checking
#ifndef LOOKOUT 
  ~StringArray();           //destructor cleans up memory
  Lstring& operator[](int); //subscript returns Lstring element
  const Lstring& operator[](int) const; //const version of subscript

private:
  Lstring *element; //pointer to a block of Lstring
  int sz;	    //size or capacity 	
#endif
};

// sub(i) macro calls _sub member function,
//   passing in information about current __FILE__ and __LINE__
//   for better error message if subscript is out of bounds
//   Lookout converts operator[] into subscript(i)
#define sub(i) _sub(__FILE__, __LINE__, (i))

#endif
