#ifndef NOTE_H  
#define NOTE_H 
/*--------- Musical Note class supplied with Lookout for C++-- --------------
Create & play sounds on computer speaker as to notes on scale.
	Hides the details of getting a computer to play musical notes!
Besides the value on the scale, each Note has its own values for octave
      relative to middle C and duration (whole, half, quarter, etc.).
Constructor sets the name, octave and playing frequency of an instance:
	Note C("C") constructs middle C; 
	Note Csh("C+") constructs middle C sharp;
	Note Eflat("E-",3) constructs E flat an octave below middle, etc.
Member function play() plays the Note's current frequency and duration:
	C.play() plays middle C as a whole note (the default duration).
Member function show() displays the Note's name, octave and duration:
	C.show(); ==> "C whole"; Eflat.show(); ==> "Eflat -1 whole".
Member function setOctave(int) can change the octave:
	C.octave(2); C.play(); C.show(); ==> "C2 whole". 
Member functions whole(),half(),quarter() and eighth() change duration:
	Csh.half(); Csh.show(); ==> "Csharp half" (half duration of whole).
Member function dot() adds half the duration to current value:
	Csh.dot(); Csh.show(); ==> "Csharp half dot"
Member function triplet alters duration for a triplet duration.
Author: Jesse Thilo, March 1995, for The Universal Machine
-----------------------------------------------------------------------------*/
#include <iostream.h>
#include "..\libfiles\Lstring.h"

class Note {
public:
  Note(Lstring,int=4); //Lstring sets note, int sets octave (see above)
  void play();         //play this Note through speaker now
  void show();         //show this Note's name, octave and duration
  void setOctave(int); //change octave of this Note, where 4 is middle C
  int getOctave();     //return octave value of the Note
  void whole();        //set duration for a while note
  void half();        //set duration for a half note
  void quarter();      //set duration for a quarter note
  void eighth();       //set duration for an eighth note
  void dot();          //set duration to current duration * 1.5
  void triplet();      //set duration for triplet: current * 2 / 3
#ifndef LOOKOUT //LOOKOUT doesn't understand the following; but C++ does
private:
  char* name;           // e.g. "C", "F+" (for f sharp), "B-" (for b flat)
  int octave;           // 4 = middle octave (A4 = concert A, 440 Hz)
  float duration;       // 1.0 = whole note, 0.5 = half note, etc.
  int frequency;        // frequency in Hz, precomputed for speed
  int note;             // index into frequency table
  void compute_frequency(void);  //computes frequency for a Note
#endif //LOOKOUT
};
#endif //NOTE_H
