#ifndef RANDOM_H  // Include header file just once
#define RANDOM_H

#include "..\libfiles\lookout.h"

/*----------- Randomizer class --------------------------------------------
  Randomizer class generates random integers or floating point numbers
  Authors: Jesse Thilo and Glenn Blank, Nov. 1995
-------------------------------------------------------------------------*/

#define MAXRANDOM 0x7FFF

class Randomizer {
public:
  Randomizer(long seed = 0);	// seed the random number generator
				// seed = 0 means use system clock
                                // each seed corresponds to a different
                                // sequence of random numbers, i.e. to
                                // reproduce a set of results, use the
                                // same seed
  void reseed(long seed = 0);	// reseed the randomizer
  int random();			// return a random integer in the
                                // range [0,MAXRANDOM]
  int randomUpTo(int range);    // return a random integer in the
                                // range [0,range)
  double random0to1(void);	// return a random floating point number
                                // in the range [0,1)
#ifndef LOOKOUT
private:
  long rand(void);              // internal random number generator
  long randtbl[32], *fptr, *rptr, *state, *end_ptr;
                                // implementation variables
#endif
};
#endif
