/*--------------------------- shapes.cpp ------------------------------
  Implementation of LOOKOUT shape classes.  The implementation details
    of drawing these shapes on a PC's screen are encapusulated here.
  Programmer: Glenn D. Blank
  For chapter 5 and following of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
---------------------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <math.h>
#include <conio.h>
//For Borland C++
#include <graphics.h>
#include "../libfiles/shapes.h"

static bool graphics_mode = false;

void TextMode(void)
{
  if (graphics_mode) {
#ifdef __BORLANDC__
    closegraph();
#else
    GrSetMode(GR_default_text);
#endif
    graphics_mode = false;
  }
}

/*
 * Protect the user from worrying about initializing and shutting down
 * graphics mode.  The first time a graphics operation is requested, the
 * screen will be initialized.  When the program terminates, graphics
 * mode is automatically shut down.
 */
void startGraphics(void)
{
  if (graphics_mode == false) {
#ifdef __BORLANDC__
    int gdriver = VGA, gmode = VGAHI, errorcode;
    errorcode = registerbgidriver(EGAVGA_driver);
    if (errorcode < 0) {
      printf("Graphics error: %s\n",
	     grapherrormsg(errorcode));
      printf("Press any key to halt\n");
      getch();
      exit(1);
    }
    initgraph(&gdriver, &gmode, "");
    errorcode = graphresult();
    if (errorcode != grOk) {
      printf("Graphics error: %s\n",
	     grapherrormsg(errorcode));
      printf("Press any key to halt\n");
      getch();
      exit(1);
    }
#else
    GrSetMode(GR_default_graphics);
#endif
    atexit(TextMode);
    graphics_mode = true;
  }
}

Point::Point(int X, int Y)
{
  color = 15;
  move(X, Y);
}

int Point::X(void)
{
  return Xcoord;
}

int Point::Y(void)
{
  return Ycoord;
}

void Point::draw()
{
#ifdef __BORLANDC__

#else
  GrPlot(Xcoord, Ycoord, color);
#endif
}

void Point::move(int X, int Y)
{
  Xcoord = X;
  Ycoord = Y;
}

Line::Line(int X1, int Y1, int X2, int Y2)
{
  color = 15;
  P1 = Point(X1, Y1);
  P2 = Point(X2, Y2);
}

double Line::length(void)
{
  int DX = P2.X() - P1.X();
  int DY = P2.Y() - P1.Y();
  return sqrt(DX * DX + DY * DY);
}

void Line::draw()
{
  startGraphics();
#ifdef __BORLANDC__
  
#else
  GrLine(P1.X(), P1.Y(), P2.X(), P2.Y(), color);
#endif
}

Circle::Circle(float rad, int X, int Y)
{
  color = 15;
  radius = rad;
  if (rad < 0.0) {
    cerr << "Circle created with negative radius:\n"
         << "  Circle(" << rad << ", " << X << ", " << Y << ")"
         << endl;
    radius = 1.0;
  }
  center.move(X, Y);
}

float Circle::perimeter(void)
{
  return 2.0 * M_PI * radius;
}

float Circle::area(void)
{
  return M_PI * radius * radius;
}

void Circle::draw(void)
{
  startGraphics();
#ifdef __BORLANDC__
  circle(center.X(), center.Y(), (int)(75 * radius));
#else
  GrCircle(center.X(), center.Y(), (int)(75 * radius), color);
#endif
  // assume 75 DPI for video screen
}

void Circle::move(int X, int Y)
{
  center.move(X, Y);
}

Triangle::Triangle(float len, int X, int Y)
{
  color = 15;
  length = len;
  if (len < 0.0) {
    cerr << "Triangle created with negative edge length:\n"
         << "  Triangle(" << len << ", " << X << ", " << Y << ")"
         << endl;
    length = 1.0;
  }
  center.move(X, Y);
}

float Triangle::perimeter(void)
{
  return 3.0 * length;
}

float Triangle::area(void)
{
  return 0.25 * length * length * sqrt(3.0);
}

void Triangle::draw(void)
{
#ifdef __BORLANDC__
  int vertices[8];
#else
  int vertices[3][2];
#endif
  int radius;

  startGraphics();
  radius = (int)(length * sqrt(3.0) * 25.0);
  // assume 75 DPI for video screen
#ifdef __BORLANDC__
  vertices[0] = vertices[6] = center.X();
  vertices[1] = vertices[7] = center.Y() - radius;
  vertices[2] = (int)(center.X() - radius / 2 * sqrt(3.0));
  vertices[4] = (int)(center.X() + radius / 2 * sqrt(3.0));
  vertices[3] = vertices[5] = center.Y() + radius / 2;
  drawpoly(4, vertices);
#else
  vertices[0][0] = center.X();
  vertices[0][1] = center.Y() - radius;
  vertices[1][0] = (int)(center.X() - radius / 2 * sqrt(3.0));
  vertices[2][0] = (int)(center.X() + radius / 2 * sqrt(3.0));
  vertices[1][1] = vertices[2][1] = center.Y() + radius / 2;
  GrPolygon(3, vertices, color);
#endif
}

void Triangle::move(int X, int Y)
{
  center.move(X, Y);
}

Rectangle::Rectangle(float hi, float wide, int X, int Y)
{
  color = 15;
  height = hi;
  width = wide;
  if (hi < 0.0 || wide < 0.0) {
    cerr << "Rectangle created with negative dimension(s):\n"
         << "  Rectangle(" << hi << ", " << wide << ", "
         << X << ", " << Y << ")"
         << endl;
    if (hi < 0.0) height = 1.0;
    if (wide < 0.0) width = 1.0;
  }
  center.move(X, Y);
}

float Rectangle::perimeter(void)
{
  return 2.0 * (height + width);
}

float Rectangle::area(void)
{
  return height * width;
}

void Rectangle::draw(void)
{
  int hi, wide;

  hi = (int)(75 * height);
  wide = (int)(75 * width);
  // assume 75 DPI for video screen
  startGraphics();
#ifdef __BORLANDC__
  rectangle(center.X() - wide / 2, center.Y() - hi / 2,
	    center.X() + wide / 2, center.Y() + hi / 2);
#else
  GrBox(center.X() - wide / 2, center.Y() - hi / 2,
        center.X() + wide / 2, center.Y() + hi / 2,
        color);
#endif
}

void Rectangle::move(int X, int Y)
{
  center.move(X, Y);
}
