/*---------------- subconv.cc: subscript conversion ------------------
  subconv is the subscript conversion program that the LOOKOUT IDE
  invokes when you press Shift-F4 or from the Lookout menu.  It
  converts any [subscript-expr] into sub(subscript-expr).  The
  LOOKOUT array and Lstring classes in turn have sub() functions
  which in turn call a _sub() macro which prints out the file
  and line numbers in the student's SOURCE code (not the array class).
  Pretty hairy!
  It is also an interesting, albeit advanced example, for students
  to study.  It uses classes Lookout class library: Lstring and InFile.
  On the advanced side, it uses parameters of main: argc and argv.  
  It does handle several special cases, to ignore '[' in strings
  or // comments.  However, it also makes some simplfying assumptions,
  such as ruling out subscript expressions over multiple lines or
  and ignoring nested subscript expressions.  (If necessary, a
  student can always run subconv.cc on its output.)
  The acid test for getting this program to work was to make sure
  it could convert subconv.cc!
  Programmer: Glenn D. Blank  
  For chapter 11 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
--------------------------------------------------------------------*/
#include "file.h"
#include <iostream.h>

bool scanner(InFile & in)
{ 
  Lstring inLine(256,'\0'); 	//holds a line of input
  Lstring outLine(256,'\0');	//holds a line of output
  int i=0; int j=0;		//counters for inLine and outLine
  while (!in.eof())
  { inLine = in.readLn();	//read a line of input
    while (i < inLine.length()) //up to end of line
    { if (inLine[i] == '/' && ((i+1) < inLine.length())
		&& inLine[i+1]=='/')  	 //Allow //...[...
      { outLine[j++] = inLine[i++];
        while (i < inLine.length())
           outLine[j++] = inLine[i++];
      }
      else if (inLine[i] == '\\')		//Allow escape chars
      { outLine[j++]=inLine[i++]; outLine[j++]=inLine[i++]; }
      else if (inLine[i] == '\"')		//Allow "//...[..."
      { outLine[j++] = inLine[i++];
        while (i < inLine.length() && inLine[i] != '\"')
           outLine[j++] = inLine[i++];
      }
      else if (inLine[i] == '\'')	//Allow '['
      { outLine[j++] = inLine[i++];
        while (i < inLine.length() && inLine[i] != '\'')
           outLine[j++] = inLine[i++];
      }
      else if (inLine[i] != '[')     //look for open subscript bracket
        outLine[j++] = inLine[i++];  //copy other characters
      else //found '[', copy subscript-expression
      { outLine[j++]='.'; outLine[j++]='s'; //output ".sub("
	outLine[j++]='u'; outLine[j++]='b'; outLine[j++]='(';
	i++;				//skip '[' in inLine
	while (i < inLine.length() && inLine[i] != ']')
	  outLine[j++] = inLine[i++];   //copy subscript expression
	if (i == inLine.length())	//reached end of line?
	{ cout << "Line " << in.getLineNumber() << ":" <<
	    "subconv needs [subscript-expression] on one line" << endl;
	  return false;
	}
	outLine[j++]=')';  		//close sub() function
	i++;				//skip ']' in inLine
      } //found '['
    } //while not end of line
    outLine[j]='\0'; 			//mark end of line
    cout << outLine << endl;		//done with current line
    i=j=0;				//reset counters for new line
  }
  return true;
}

int main(int argc, char *argv[])
{
  if (argc == 1)
  { cout << "Usage: subconv <filename>" << endl;
    return 1;
  }
  InFile in(argv[1]);
  if (!in.exists())
  { cout << "Cannot open file <filename>" << endl;
    return 1;
  }
  bool result = scanner(in);  //did scanner succeed?
  in.close();
  if (result==true) return 0;
  else return 1;
}

